# ----------------------------------------------------------------------
# Curses::UI::Language::portuguese
# Maintainer: raul@dias.com.br
# ----------------------------------------------------------------------

package Curses::UI::Language::portuguese; 
1;
__DATA__

# ----------------------------------------------------------------------
# For Curses::UI
# ----------------------------------------------------------------------

[screen_too_small]
A sua tela atualmente é muito pequena para essa aplicação.
Altere o tamanho da tela e reinicie a aplicação.
Pressione <CTRL+C> para sair...

# ----------------------------------------------------------------------
# For Curses::UI::Calendar
# ----------------------------------------------------------------------

[months]
Janeiro Fevereiro Março Abril Maio Junho Julho
Agosto Setembro Outubro Novembro Dezembro

[days_short] Do Se Te Qa Qi Se Sa

# ----------------------------------------------------------------------
# For Curses::UI::Buttonbox
# ----------------------------------------------------------------------

[button_ok]      o:OK
[button_cancel]  c:Cancelar
[button_yes]     s:Sim
[button_no]      n:Não

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::Error
# ----------------------------------------------------------------------

[error_title]    Mensagem de Erro

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::FileBrowser
# ----------------------------------------------------------------------

# The filebrowser title
[file_title]     Selecione um arquivo
[file_savetitle] Selecione um arquivo para salvar
[file_loadtitle] Selecione um arquivo para carregar

# The labels for the dialog screen.
[file_path]      Caminho :
[file_file]      Arquivo :
[file_mask]      Mascara :

# The size of the longest label
[file_labelsize] 9

# On directory up
[file_dirup] Diretório Anterior

# For asking the user if a file may be overwritten
[file_overwrite_title] Confirme
[file_overwrite_question_pre] 
Você quer realmente sobrescrever esse arquivo "
[file_overwrite_question_post] 
"?

# Errors
[file_err_opendir_pre] Não posso abrir o diretório "
[file_err_opendir_post] "
[file_err_nofileselected] Você não selecionou um arquivo!


