# ----------------------------------------------------------------------
# Curses::UI::Language::spanish
# Maintainer: marcus@thiesenweb.de
# ----------------------------------------------------------------------

package Curses::UI::Language::spanish;
1;
__DATA__

# ----------------------------------------------------------------------
# For Curses::UI
# ----------------------------------------------------------------------

[screen_too_small]
Tu pantalla es muy pequeña para esta aplicación.
Cambia el tamaño de tu pantalla y reinicia la aplicación.
Presiona <CTRL+C> para salir...

# ----------------------------------------------------------------------
# For Curses::UI::Calendar
# ----------------------------------------------------------------------

[months]
Enero Febrero Marzo Abril Mayo Junio Julio 
Agosto Septiembre Octubre Noviembre Diciembre

[days_short] Do Lu Mar Mie Ju Vi Sa

# ----------------------------------------------------------------------
# For Curses::UI::Buttonbox
# ----------------------------------------------------------------------

[button_ok]      o:Aceptar
[button_cancel]  c:Cancelar
[button_yes]     y:Sí
[button_no]      n:No

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::Error
# ----------------------------------------------------------------------

[error_title]    Mensaje de Error

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::FileBrowser
# ----------------------------------------------------------------------

# The filebrowser title
[file_title]     Selecciona un archivo
[file_savetitle] Selecciona un archivo a donde grabar
[file_loadtitle] Selecciona un archivo de donde cargar

# The dirbrowser title
[dir_title]     Selecciona un directorio

# The labels for the dialog screen.
[file_path]      Ruta :
[file_file]      Archivo :
[file_mask]      Máscara :

# The size of the longest label
[file_labelsize] 6

# On directory up
[file_dirup] Directory padre

# For asking the user if a file may be overwritten
[file_overwrite_title] Confirmar
[file_overwrite_question_pre] 
Realmente quieres sobreescribir el archivo "
[file_overwrite_question_post] 
"?

# Errors
[file_err_opendir_pre] No se puede abrir el directorio "
[file_err_opendir_post] "
[file_err_nofileselected] No has seleccionado un archivo!

